package com.loafle.overflow.probe.container.server;

import io.netty.bootstrap.ServerBootstrap;
import io.netty.channel.*;
import io.netty.channel.nio.NioEventLoopGroup;
import io.netty.channel.socket.nio.NioServerSocketChannel;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;

public class ContainerServer {
  private int port;

  public ContainerServer() {
  }

  public void start(String pidPath) throws Exception {
    EventLoopGroup bossGroup = new NioEventLoopGroup(1);
    EventLoopGroup workerGroup = new NioEventLoopGroup(10);
    try {
      ServerBootstrap b = new ServerBootstrap();

      b.group(bossGroup, workerGroup)
              .channel(NioServerSocketChannel.class)
              .childHandler(ContainerConfiguration.channelInitializer())
              .option(ChannelOption.SO_BACKLOG, 128)
              .childOption(ChannelOption.SO_KEEPALIVE, true);

      ChannelFuture f = null;

      for (int i = 60000; i < 61000; i++) {
        try {
          f = b.bind("127.0.0.1", i).sync();
          this.port = i;
          break;
        } catch (Exception e) {
          System.out.println(e);
          continue;
        }
      }

      if (null == f) {
        System.out.println("There is not available port");
        return;
      }

      try {
        FileOutputStream outputStream = new FileOutputStream(pidPath);
        outputStream.write(Integer.toString(this.port).getBytes());
        outputStream.close();
      } catch (IOException e) {
        System.out.println(e);
        return;
      }

      Runtime.getRuntime().addShutdownHook(new Thread()
      {
        @Override
        public void run()
        {
          ContainerServer.onStop(pidPath);
        }
      });

      f.channel().closeFuture().sync();
    } finally {
      onStop(pidPath);

      workerGroup.shutdownGracefully();
      bossGroup.shutdownGracefully();
    }
  }

  private static void onStop(String pidPath) {
    try {
      File f = new File(pidPath);
      f.delete();
    } catch (Exception e) {
      System.out.println(e);
    }
  }

}
